#!/usr/bin/env python
from elastictex import api
from flask import Flask, request, render_template

app = Flask(__name__)


def main():
    api.init()
    app.run(debug=True)


@app.route('/')
def get():
    results = None
    q = request.args.get('q')
    if q:
        results = api.search(q)
    return render_template("index.html",
                           results=results)


@app.route('/upload', methods=['POST'])
def post():
    url = request.args.get('url')
    if url is None:
        return "Invalid URL"
    else:
        api.index(url, request.data)
        return "Indexed %s\n" % url


if __name__ == "__main__":
    main()



import re

from bs4 import BeautifulSoup
from urlparse import urlparse
from elasticsearch import Elasticsearch


var_types = {
    "integer_constants": {
        'name': 'K',
        'value': [
            'i', 'j', 'k', 'n', 'm'
        ]
    },
    "continuous_constants": {
        'name': 'C',
        'value': [
            'a', 'b', 'c', 'p', 'q', 's', 't'
        ]
    },
    "variables": {
        "name": "X",
        'value': [
            'x', 'y', 'z', 'u', 'v', 'w'
        ]
    }
}
var_mapping = {}
for x in var_types.values():
    for val in x['value']:
        var_mapping[val] = x['name']


es = Elasticsearch(hosts=["localhost:9200"])


def init():
    es.indices.create(index="math", ignore=400)
    mapping = {
        "equation": {
            "properties": {
                "processed": {
                    "type": "string",
                    "index": "not_analyzed"
                }
            }
        }
    }
    es.indices.put_mapping(index="math",
                           doc_type="equation",
                           body=mapping)


def search(s=None):
    term = canonicalize(s)
    print(term)
    query = {
        "query": {
            "in": {
                "processed": [term]
            }
        }
    }
    result = es.search(index="math", doc_type="equation", body=query,
                       size=20)

    docs = []
    for hit in result['hits']['hits']:
        doc = hit['_source']
        docs.append(doc)
    return docs


def store(title, url, equations):
    for eq in equations:
        processed = [canonicalize(x) for x in expand(eq)]
        doc = {
            "title": title,
            "url": url,
            "equation": eq,
            "processed": processed
        }
        es.index(index="math", doc_type="equation", body=doc)


def index(url, html):
    info = urlparse(url)
    domain = info.scheme + '://' + info.netloc
    bs = BeautifulSoup(html)
    title = get_title(bs)

    question = bs.find("div", {"class": "question"})
    store(title, url, get_equations(question))

    answers = bs.find_all("div", {"class": "answer"})
    for i, answer in enumerate(answers):
        answer_url = "%s/a/%s" % (
            domain, answer['id'].split('-')[1])
        store(title + "(answer %d)" % (i+1),
              answer_url, get_equations(answer))


def get_title(element):
    title = element.find("title").text
    parts = title.split('-', 1)
    if parts[0] == parts[0].lower():
        title = parts[1]
    return title.rsplit('-', 1)[0]


def get_equations(element):
    equations = []
    if element:
        pars = element.find_all("p")
        for par in pars:
            equations.extend(
                text for i, text in enumerate(par.text.split("$"))
                if i % 2 == 1 and len(text) > 12)
    return equations


def get_var_name(c):
    for x in var_types.values():
        if c in x['value']:
            return x['name']
    return c.upper()


def is_constant(name):
    for constant in constants:
        if name.startswith(constant):
            return True
    return False


def replace_vars(s):
    # Replace Variables
    var_re = re.compile(r"[a-z](_[0-9])?")
    counts = {}
    start = 0
    while True:
        m = var_re.search(s, start)
        if m is None:
            break

        name = m.group()
        var_name = var_mapping.get(name[:1], name.upper())
        count = counts.get(var_name, 0)
        s = re.sub(r"%s" % name,
                   "[%s%d]" % (var_name, count),
                   s)
        counts[var_name] = count + 1

    return s


def expand(s):
    parts = [s]
    stack = []
    expand_braces(parts, stack, s, 0)

    def split(c):
        n = len(parts)
        for i in range(n):
            if c in parts[i]:
                parts.extend(parts[i].split(c))

    split('=')
    split('+')
    split('-')

    return parts


def expand_braces(result, stack, s, i):
    expr = re.compile(r"[\{\}=]")
    m = expr.search(s, i)
    if not m:
        result.append(s)
        return len(s)

    c = m.group()
    j = m.start()

    if c == "}":
        if stack:
            result.append(s[stack.pop():j])
        return j+1

    if c == "{":
        result.append(s[i:j])
        stack.append(j+1)
        i = expand_braces(result, stack, s, j+1)
        if i < 0:
            i = len(s)
        return expand_braces(result, stack, s, i)


def canonicalize(s):
    # turn \sqrt -> [\SQRT]
    def sub_macro(match):
        return "[" + match.group(1).upper() + "]"
    s = re.sub(r"(\\\w+\d*)\b", sub_macro, s)

    # turn x_{1} -> x_1
    s = re.sub(r"_\{(\d)\}", r"_\1", s)

    # replace variables with canonical names
    s = replace_vars(s)

    # make sure single digit arguments are converted
    s = re.sub(r"\^(\d)", r"^{\1}", s)
    s = re.sub(r"\_(\d)", r"_{\1}", s)

    # remove spaces
    s = s.replace(" ", "")

    return s



import urllib
import logging
from urllib2 import urlopen, Request
from mongo_connector.doc_managers.doc_manager_base import DocManagerBase

LOG = logging.getLogger(__name__)


class DocManager(DocManagerBase):
    def __init__(self, url=None, **kwargs):
        self.url = url
        if self.url is None:
            self.url = "localhost:5000"

    def upsert(self, doc):
        url = "%s/upload?url=%s" % (self.url, urllib.quote(str(doc['_id'])))
        if not (url.startswith("http://") or url.startswith("https://")):
            url = "http://" + url

        req = Request(url)
        req.add_header("Content-type", "application/octet-stream")
        req.add_data(doc['body'].encode('utf-8'))
        response = urlopen(req)
        LOG.info(response.read())

    def update(self, doc, update_spec):
        pass

    def remove(self, doc):
        pass

    def search(self, start_ts, end_ts):
        pass

    def commit(self):
        pass

    def get_last_doc(self):
        pass

    def stop(self):
        pass



#!/usr/bin/env python

# TODO: disallow tag urls
# TODO: instead of done set and queue_set, just query mongo for existence

import traceback
import random
from Queue import Queue
from bs4 import BeautifulSoup
from urllib2 import urlopen
from urlparse import urlparse
from pymongo import MongoClient
from pymongo.errors import AutoReconnect

queue = Queue()
queue_set = set()

starters = [
    "http://math.stackexchange.com/questions/309267/quadratic-equation"
]

client = MongoClient()
data = client['crawler']['data']


def main():
    if data.count() > 1:
        print("Recovering from last crawling session.")
        maxlevel = data.find_one(sort=[("$natural", -1)])['level']
        docs = data.find({'level': maxlevel - 1},
                         sort=[("rand", 1)], limit=25)
        for doc in docs:
            add_links(doc['_id'], doc['body'], doc['level'])
    else:
        for starter in starters:
            queue.put((starter, 0))

    # Start the crawl loop
    while not queue.empty():
        (url, level) = queue.get()

        crawl(url, level)


def clean_link(link):
    if urlparse(link).netloc:
        return None
    if "questions" not in link:
        return None
    if "ask" in link:
        return None
    if "login" in link:
        return None
    if "tagged" in link:
        return None
    if "?" in link:
        link = link.split("?", 1)[0]
    return link


def add_links(url, body, level):
    url_struct = urlparse(url)
    html = BeautifulSoup(body)
    links = [a.get('href') for a in html.find_all("a")]
    for link in links:
        if not link:
            continue

        link = clean_link(link)
        if link:
            full = url_struct.scheme + "://" + url_struct.netloc + link
            if full:
                queue.put((full, level+1))


def crawl(url, level):
    try:
        doc = data.find({'_id': url}, {'_id': 1})
        if doc.count() > 0:
            print("Skipped %s (%d)" % (url, level))
            return

        body = urlopen(url).read()
        add_links(url, body, level)

        data.update({
            '_id': url
        }, {
            'level': level,
            'body': body,
            'rand': random.random()
        }, upsert=True)
        print("Crawled %s (%d)" % (url, level))

    except KeyboardInterrupt:
        print("Bye Bye")
        exit(0)

    except AutoReconnect:
        print("AutoReconnect")
        crawl(url, level)

    except:
        traceback.print_exc()

if __name__ == "__main__":
    main()
